--[[
	bar.lua
		Scripts used for the Bongos Bag bar

	Saved Variables:
		Bongos.bag = {
			<All variables from BBar>
			space
				The spacing between action buttons, in pixels.  A nil value means that the bar is using default spacing
			rows
				How many rows the bar is organized into.
			oneBag
				Flag for if we're only showing the main bag.
		}
--]]

--constants
local DEFAULT_SPACING = 4;
local DEFAULT_ROWS = 1;

function BBagBar_Create()
	local bar = BBar_Create("BBagBar", "bags");
	BBagBarDragButton:SetScript("OnClick", BBagBarDragButton_OnClick);
	
	CharacterBag3Slot:SetParent(BBagBar);
	CharacterBag2Slot:SetParent(BBagBar);
	CharacterBag1Slot:SetParent(BBagBar);
	CharacterBag0Slot:SetParent(BBagBar);
	MainMenuBarBackpackButton:SetParent(BBagBar);
	
	local alpha = BBagBar:GetAlpha();
	if(alpha ~= 1) then
		CharacterBag3Slot:SetAlpha(alpha);
		CharacterBag2Slot:SetAlpha(alpha);
		CharacterBag1Slot:SetAlpha(alpha);
		CharacterBag0Slot:SetAlpha(alpha);
		MainMenuBarBackpackButton:SetAlpha(alpha);
	end
	
	if(not BBagBar:IsUserPlaced() ) then
		BBagBar:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT");
	end
	
	if( BBagBar:IsShown() ) then
		BBagBar:Hide();
		BBagBar:Show();
	end
	
	BBagBar_Build(Bongos.bags.rows, Bongos.bags.space);
	return bar;
end

function BBagBarDragButton_OnClick()
	if(arg1=="RightButton") then
		BBagBar_ShowMenu();
	end
end

--[[ Loading Functions ]]--
function BBagBar_Load()
	BBagBar_Create();
end

--[[ UI Functions ]]--

--Layout bar
function BBagBar_Build(rows, space)
	if(not rows or rows == DEFAULT_ROWS) then
		Bongos.bags.rows = nil;
		rows = nil;
	else
		Bongos.bags.rows = rows;
	end

	if(not space or space == DEFAULT_SPACING) then
		Bongos.bags.space = nil;
		space = DEFAULT_SPACING;
	else
		Bongos.bags.space = space;
	end
	
	CharacterBag3Slot:ClearAllPoints();
	CharacterBag2Slot:ClearAllPoints();
	CharacterBag1Slot:ClearAllPoints();
	CharacterBag0Slot:ClearAllPoints();
	MainMenuBarBackpackButton:ClearAllPoints();
	
	if(Bongos.bags.oneBag) then
		for i = 0, 3 do
			getglobal("CharacterBag" .. i .. "Slot"):Hide();
		end
		
		MainMenuBarBackpackButton:SetPoint("TOPLEFT", BBagBar, "TOPLEFT", 2, -1);
		BBagBar:SetWidth(37);
		BBagBar:SetHeight(37);
	else
		for i = 0, 3 do
			getglobal("CharacterBag" .. i .. "Slot"):Show();
		end
		
		--vertical alignment
		if(rows) then
			CharacterBag3Slot:SetPoint("TOPLEFT", BBagBar, "TOPLEFT", 2, -1);
			CharacterBag2Slot:SetPoint("TOP", CharacterBag3Slot, "BOTTOM", 0, -1 * space);
			CharacterBag1Slot:SetPoint("TOP", CharacterBag2Slot, "BOTTOM", 0, -1 * space);
			CharacterBag0Slot:SetPoint("TOP", CharacterBag1Slot, "BOTTOM", 0, -1 * space);
			MainMenuBarBackpackButton:SetPoint("TOP", CharacterBag0Slot, "BOTTOM", 0, -1 * space);
			
			BBagBar:SetWidth( (37 + space ) - space + 3);
			BBagBar:SetHeight( (37 + space ) * 5 - space + 3);
		--horizontal alignment
		else
			CharacterBag3Slot:SetPoint("TOPLEFT", BBagBar, "TOPLEFT", 2, -1);
			CharacterBag2Slot:SetPoint("LEFT", CharacterBag3Slot, "RIGHT", space, 0);
			CharacterBag1Slot:SetPoint("LEFT", CharacterBag2Slot, "RIGHT", space, 0);
			CharacterBag0Slot:SetPoint("LEFT", CharacterBag1Slot, "RIGHT", space, 0);
			MainMenuBarBackpackButton:SetPoint("LEFT", CharacterBag0Slot, "RIGHT", space, 0);
			
			BBagBar:SetWidth((37 + space ) * 5 - space + 3);
			BBagBar:SetHeight((37 + space ) - space + 3);
		end
	end
end

--[[ Rightclick Menu Functions ]]--

local function CreateConfigMenu(name)
	local menu = CreateFrame("Button", name, UIParent, "BongosRightClickMenuHide");
	menu:SetText("Bag Bar");
	menu:SetWidth(220);
	menu:SetHeight(250);
	
	local oneBagButton = CreateFrame("CheckButton", name .. "OneBag", menu, "BongosCheckButtonTemplate");
	oneBagButton:SetScript("OnClick", function()
		BBagBar_ShowAsOneBag( this:GetChecked() );
	end);
	oneBagButton:SetPoint("TOP", name .. "Hide", "BOTTOM", 0, 4);
	oneBagButton:SetText("One Bag");
	
	local verticalButton = CreateFrame("CheckButton", name .. "Vertical", menu, "BongosCheckButtonTemplate");
	verticalButton:SetScript("OnClick", function()
		if( this:GetChecked() ) then
			BBagBar_Build(5, Bongos.bags.space );
		else
			BBagBar_Build(nil, Bongos.bags.space );
		end
	end);
	verticalButton:SetPoint("TOP", name .. "OneBag", "BOTTOM", 0, 4);
	verticalButton:SetText("Vertical");
	
	local spacingSlider = CreateFrame("Slider", name .. "Spacing", menu, "BongosSpaceSlider");
	spacingSlider:SetPoint("TOPLEFT", name .. "Vertical", "BOTTOMLEFT", 2, -10);
	spacingSlider:SetScript("OnValueChanged", function()
		if(not this:GetParent().onShow) then
			BBagBar_Build(Bongos.bags.rows, this:GetValue() );
		end
		getglobal(this:GetName() .. "ValText"):SetText( this:GetValue() );
	end);
	
	local scaleSlider = CreateFrame("Slider", name .. "Scale", menu, "BongosScaleSlider");
	scaleSlider:SetPoint("TOP", spacingSlider, "BOTTOM", 0, -24);
	
	local opacitySlider = CreateFrame("Slider", name .. "Opacity", menu, "BongosOpacitySlider");
	opacitySlider:SetPoint("TOP", scaleSlider, "BOTTOM", 0, -24);
end

--Called when the right click menu is shown, loads the correct values to the checkbuttons/sliders/text
function BBagBar_ShowMenu()
	if(not BongosBagBarMenu) then
		CreateConfigMenu("BongosBagBarMenu");
	end
	
	BongosBagBarMenu.onShow = 1;
	BongosBagBarMenu.frame = this:GetParent();
	
	BongosBagBarMenuHide:SetChecked( not  Bongos.bags.vis);
	BongosBagBarMenuOneBag:SetChecked(Bongos.bags.oneBag);
	BongosBagBarMenuVertical:SetChecked( Bongos.bags.rows);
	
	if(Bongos.bags.space) then
		BongosBagBarMenuSpacing:SetValue(Bongos.bags.space);
	else
		BongosBagBarMenuSpacing:SetValue(DEFAULT_SPACING);
	end
	
	BongosBagBarMenuScale:SetValue( BBagBar:GetScale() * 100 );
	BongosBagBarMenuOpacity:SetValue( BBagBar:GetAlpha() * 100 );
	
	BMenu_ShowMenuForBar(BongosBagBarMenu, BBagBar);
	BongosBagBarMenu.onShow = nil;
end

function BBagBar_ShowAsOneBag(enable)
	if(enable) then
		Bongos.bags.oneBag = 1;
	else
		Bongos.bags.oneBag = nil;
	end
	BBagBar_Build(Bongos.bags.rows, Bongos.bags.space);
end