--[[
	unreg
		Handles hiding/unregistering of any blizzard UI component replaced by Bongos
		
	Frames Unregistered:
		MainMenuBar
		Experience Bar
		Action Bar
		Multibars
		Bonusbars
		Shapeshift
		Pet
		GroupLoot
			
	TODO:
		Unregister any unnecessary events
--]]

local UnregisterActionButton, UnregisterActionBars, UnregisterBonusBar, UnregisterShapeshiftBar, UnregisterPetBar;

--Should be called by BMain
function Bongos_UnregisterAll()
	MainMenuBar:Hide();
	ExhaustionTick:UnregisterAllEvents();
	
	UnregisterActionBars();
	UnregisterShapeshiftBar();
	UnregisterPetBar();
end

--[[ Unregistering Functions ]]--

--Unregister action buttons
function UnregisterActionButton(button)
	button:UnregisterAllEvents();
	button:Hide();
end

--Hide action bar
function UnregisterActionBars()
	BonusActionBarFrame:Hide();
	
	--Action Buttons
	for i = 1, 12, 1 do
		--main action bar
		UnregisterActionButton( getglobal("ActionButton"..i) );
		getglobal("ActionButton"..i).showgrid = nil;
		getglobal("ActionButton"..i).flashing = nil;
		getglobal("ActionButton"..i).flashtime = nil;
		
		--multibars
		UnregisterActionButton( getglobal("MultiBarBottomLeftButton"..i) );
		UnregisterActionButton( getglobal("MultiBarBottomRightButton"..i) );
		UnregisterActionButton( getglobal("MultiBarLeftButton"..i) );
		UnregisterActionButton( getglobal("MultiBarRightButton"..i) );
		
		--bonus bar
		UnregisterActionButton( getglobal("BonusActionButton"..i) );
	end	
	
	ActionBarUpButton:UnregisterEvent("ACTIONBAR_PAGE_CHANGED");
	BonusActionBarFrame:UnregisterAllEvents();
end

--Hide shapeshift bars
function UnregisterShapeshiftBar()
	ShapeshiftBarFrame:UnregisterAllEvents();
end

--Hide pet bar
function UnregisterPetBar()
	PetActionBarFrame:UnregisterAllEvents();
	PetActionBarFrame:Hide();
	PetActionBarFrame.showgrid = nil;
	
	for i=1, NUM_PET_ACTION_SLOTS do
		getglobal("PetActionButton" .. i):UnregisterAllEvents();
	end
end