--[[
	bar.lua
		Scripts used for the Bongos key bar

	Saved Variables:
		Bongos.keys = {
			<All variables from BBar>
			space
				The spacing between action buttons, in pixels.  A nil value means that the bar is using default spacing
			rows
				How many rows the bar is organized into.
		}
		
	TODO:
		One key mode.
--]]

--[[ Loading Functions ]]--
function BKeyBar_Load()
	BKeyBar_Create();
end

--constants
function BKeyBar_Create()
	local bar = BBar_Create("BKeyBar", "key");
	BKeyBar:SetWidth(20);
	BKeyBar:SetHeight(40);
	BKeyBarDragButton:SetScript("OnClick", BKeyBarDragButton_OnClick);
	
	KeyRingButton:ClearAllPoints();
	KeyRingButton:SetPoint("TOPLEFT", BKeyBar, "TOPLEFT", 1, -1);
	KeyRingButton:SetParent(BKeyBar);
	
	local alpha = BKeyBar:GetAlpha();
	if(alpha ~= 1) then
		KeyRingButton:SetAlpha(alpha);
	end
	
	if(not BKeyBar:IsUserPlaced() ) then
		BKeyBar:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -220, 0);
	end
	
	if( BKeyBar:IsShown() ) then
		BKeyBar:Hide();
		BKeyBar:Show();
	end
	
	return bar;
end

function BKeyBarDragButton_OnClick()
	if(arg1=="RightButton") then
		BKeyBar_ShowMenu();
	end
end

--[[ Rightclick Menu Functions ]]--

local function CreateConfigMenu(name)
	local menu = CreateFrame("Button", name, UIParent, "BongosRightClickMenuHide");
	menu:SetText("Key Bar");
	menu:SetWidth(220);
	menu:SetHeight(152);

	local scaleSlider = CreateFrame("Slider", name .. "Scale", menu, "BongosScaleSlider");
	scaleSlider:SetPoint("TOPLEFT", name .. "Hide", "BOTTOMLEFT", 2, -10);
	
	local opacitySlider = CreateFrame("Slider", name .. "Opacity", menu, "BongosOpacitySlider");
	opacitySlider:SetPoint("TOP", scaleSlider, "BOTTOM", 0, -24);
end

--Called when the right click menu is shown, loads the correct values to the checkbuttons/sliders/text
function BKeyBar_ShowMenu()
	if(not BongosKeyBarMenu) then
		CreateConfigMenu("BongosKeyBarMenu");
	end
	
	BongosKeyBarMenu.onShow = 1;
	BongosKeyBarMenu.frame = this:GetParent();
	
	BongosKeyBarMenuHide:SetChecked( not Bongos.key.vis);
	BongosKeyBarMenuScale:SetValue( BKeyBar:GetScale() * 100 );
	BongosKeyBarMenuOpacity:SetValue( BKeyBar:GetAlpha() * 100 );
	
	BMenu_ShowMenuForBar(BongosKeyBarMenu, BKeyBar);
	BongosKeyBarMenu.onShow = nil;
end