local compost = AceLibrary("Compost-2.0")
local parser = ParserLib:GetInstance("1.1")
local roster = AceLibrary("RosterLib-2.0")


local dewdrop -- Load on demand

-- Link these after all files are loaded.
local loc
local defaultFormats
local defaultColors
local settingsPerChar

-- Return a deep copy of this table.
local function CopyTable(into, from)
	for key, val in from do
		if( type(val) == "table" ) then
			if not into[key] then into[key] = {} end
			CopyTable(into[key], val)
		else
			into[key] = val
		end
	end
	return into
end


------------------------
--         Core         --
------------------------

SimpleCombatLog = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0")

--- SimpleCombatLog.modulePrototype.core = SimpleCombatLog

-- Hook the FCF_Tab_OnClick() to check for alt-rightclick.
function SimpleCombatLog:FCF_Tab_OnClick(button)
	if IsAltKeyDown() and button == 'RightButton' then
		self:LoadSettingMenu(this:GetID())
	else
		self.old_FCF_Tab_OnClick(button)
	end
end

-- Hook ChatFrame_OnEvent to suppress the default combat log.
function SimpleCombatLog:ChatFrame_OnEvent(event)
	local id = this:GetID()
	
	-- Note to self: must do ( self.events[event] == nil ) here, cannot do  ( not self.events[event] ), since some events can be 'false' by default.
	if self.events[event] == nil or not id or not self.settings[id] or not self.settings[id].suppress then
		self.old_ChatFrame_OnEvent(event)
	end
end


-- Events to filter, the flags are also used as default value for 'Load Common Settings'.
SimpleCombatLog.events = {
	['CHAT_MSG_COMBAT_PET_HITS'] = true,
	['CHAT_MSG_COMBAT_PET_MISSES'] = true,
	['CHAT_MSG_COMBAT_SELF_HITS'] = true,
	['CHAT_MSG_COMBAT_SELF_MISSES'] = true,	
	['CHAT_MSG_COMBAT_CREATURE_VS_CREATURE_HITS'] = true,
	['CHAT_MSG_COMBAT_CREATURE_VS_CREATURE_MISSES'] = true,
	['CHAT_MSG_COMBAT_CREATURE_VS_PARTY_HITS'] = true,
	['CHAT_MSG_COMBAT_CREATURE_VS_PARTY_MISSES'] = true,
	['CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS'] = true,
	['CHAT_MSG_COMBAT_CREATURE_VS_SELF_MISSES'] = true,
	['CHAT_MSG_COMBAT_FRIENDLYPLAYER_HITS'] = true,
	['CHAT_MSG_COMBAT_FRIENDLYPLAYER_MISSES'] = true,
	['CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS'] = true,
	['CHAT_MSG_COMBAT_HOSTILEPLAYER_MISSES'] = true,
	['CHAT_MSG_COMBAT_PARTY_HITS'] = true,
	['CHAT_MSG_COMBAT_PARTY_MISSES'] = true,
	['CHAT_MSG_SPELL_PET_BUFF'] = true,
	['CHAT_MSG_SPELL_PET_DAMAGE'] = true,
	['CHAT_MSG_SPELL_SELF_BUFF'] = true,
	['CHAT_MSG_SPELL_SELF_DAMAGE'] = true,
	['CHAT_MSG_SPELL_CREATURE_VS_CREATURE_BUFF'] = true,
	['CHAT_MSG_SPELL_CREATURE_VS_CREATURE_DAMAGE'] = true,
	['CHAT_MSG_SPELL_CREATURE_VS_PARTY_BUFF'] = true,
	['CHAT_MSG_SPELL_CREATURE_VS_PARTY_DAMAGE'] = true,
	['CHAT_MSG_SPELL_CREATURE_VS_SELF_BUFF'] = true,
	['CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE'] = true,
	['CHAT_MSG_SPELL_FRIENDLYPLAYER_BUFF'] = true,
	['CHAT_MSG_SPELL_FRIENDLYPLAYER_DAMAGE'] = true,
	['CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF'] = true,
	['CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE'] = true,
	['CHAT_MSG_SPELL_PARTY_BUFF'] = true,
	['CHAT_MSG_SPELL_PARTY_DAMAGE'] = true,
	['CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS'] = true,
	['CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE'] = true,
	['CHAT_MSG_SPELL_DAMAGESHIELDS_ON_OTHERS'] = true,	
	['CHAT_MSG_SPELL_DAMAGESHIELDS_ON_SELF'] = true,	
	['CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS'] = true,
	['CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE'] = true,
	['CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS'] = true,
	['CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_DAMAGE'] = true,
	['CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_BUFFS'] = true,
	['CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE'] = true,
	['CHAT_MSG_SPELL_PERIODIC_PARTY_BUFFS'] = true,
	['CHAT_MSG_SPELL_PERIODIC_PARTY_DAMAGE'] = true,
	['CHAT_MSG_SPELL_AURA_GONE_OTHER'] = true,
	['CHAT_MSG_SPELL_AURA_GONE_SELF'] = true,
	['CHAT_MSG_SPELL_AURA_GONE_PARTY'] = true,
	['CHAT_MSG_SPELL_BREAK_AURA'] = true,
	['CHAT_MSG_COMBAT_FRIENDLY_DEATH'] = true,
	['CHAT_MSG_COMBAT_HOSTILE_DEATH'] = true,
	['CHAT_MSG_SPELL_ITEM_ENCHANTMENTS'] = true,
	['CHAT_MSG_COMBAT_XP_GAIN'] = true,
	['CHAT_MSG_COMBAT_HONOR_GAIN'] = true,
	['CHAT_MSG_COMBAT_FACTION_CHANGE'] = true,
	['CHAT_MSG_SPELL_TRADESKILLS'] = true,
	['CHAT_MSG_SPELL_FAILED_LOCALPLAYER'] = false,
}

-- Available filters.
SimpleCombatLog.typeFilters = {
	'hit', 'heal', 'miss', 'cast', 'gain', 'drain', 'leech', 'dispel', 
	'buff', 'debuff', 'fade', 'interrupt', 'death', 'environment', 'extraattack', 'enchant', 
}

SimpleCombatLog.nameFilters = {
	'player', 'pet', 'party', 'raid', 'target', 'targettarget', 'other',
}


function SimpleCombatLog:OnInitialize()
	
	loc = self.loc
	defaultFormats = self.defaultFormats
	defaultColors = self.defaultColors
	
	self.old_FCF_Tab_OnClick = FCF_Tab_OnClick
	FCF_Tab_OnClick = function(button) self:FCF_Tab_OnClick(button) end
	
	self.old_ChatFrame_OnEvent = ChatFrame_OnEvent
	ChatFrame_OnEvent = function(event) self:ChatFrame_OnEvent(event) end

	-- index to Blizzard chat frames.
	self.chatFrames = { ChatFrame1, ChatFrame2, ChatFrame3, ChatFrame4, ChatFrame5, ChatFrame6, ChatFrame7 } 
	
	self:RegisterChatCommand( { "/scl" }, {
		type = 'group',
		args = {
			help = {	
				type = 'execute',
				name = 'help',
				desc = loc["CmdHelpDesc"],
				func = "ShowHelp",
			},
			reset = {
				type = 'execute',
				name = 'reset',
				desc = loc["CmdResetDesc"],
				func = "LoadInitialSettings",
			},
			show = {
				type = 'text',
				name = 'config',
				desc = loc["CmdShowDesc"],
				get = false,
				set = function(id) self:ShowSettingMenu(tonumber(id)) end,
-- 				input = true, 	-- dunno why, this isn't working.
				validate = { '1', '2', '3', '4', '5', '6', '7' },
				usage = "<ChatFrame ID>",
			}
		}
	
	} )
	

	self:InitGlobalDB()
	self:LoadThemes()

	self.settings = {}

	self:InitPerCharDB()
	self:LoadSettings()
	
	if self.firstLoad then
		self:LoadInitialSettings()
		self:ShowHelp()
	end
	
	
end

function SimpleCombatLog:InitGlobalDB()
	if SimpleCombatLogDB and SimpleCombatLogDB.version ~= self.version then
		self:Print("SCL version changed, resetting global data.")
	end
		
	if not SimpleCombatLogDB or SimpleCombatLogDB.version ~= self.version then
		SimpleCombatLogDB = {
			version = self.version,
			themes = {}
		}
	end
	
	self.themeDB = SimpleCombatLogDB.themes	
end

function SimpleCombatLog:InitPerCharDB()
	if SimpleCombatLogDBPerChar and SimpleCombatLogDBPerChar.version ~= self.version then
		self:Print("SCL version changed, resetting per-char data.")
	end
	
	if not SimpleCombatLogDBPerChar or SimpleCombatLogDBPerChar.version ~= self.version then
		self.firstLoad = true		
		SimpleCombatLogDBPerChar = {
			version = self.version,
			settings = {}
		}
	end	
	
	self.settingDB = SimpleCombatLogDBPerChar.settings
end

-- Load themes into 
function SimpleCombatLog:LoadThemes()
	for i, theme in self.themeDB do
		self.themes[i] = theme
	end	
end

function SimpleCombatLog:LoadSettings()
	for i, v in self.settingDB do
		if type(v) == 'string' then
			self.settings[i] = self.themes[v]
		else
			self.settings[i] = v
		end
	end
end

function SimpleCombatLog:ShowHelp()
	for i, msg in loc["Welcome"] do
		self:Print(msg)
	end
end

function SimpleCombatLog:OnEnable()

	self:RefreshSettings()
	roster:Enable()
end

function SimpleCombatLog:OnDisable()
	parser:UnregisterAllEvents("SimpleCombatLog")
	roster:Disable()
end


function SimpleCombatLog:OnCombatEvent(event, info)

	-- Parse for the message type.
	local infoType = info.type
	local msgType
	if infoType == 'hit' then
		if info.isCrit then 
			msgType = 'hitCrit'
		elseif info.isDOT then 
			msgType = 'hitDOT'
		else
			msgType = 'hit'
		end
	elseif infoType == 'heal' then
		if info.isCrit then
			msgType = 'healCrit'
		elseif info.isDOT then
			msgType = 'healDOT'
		else
			msgType = 'heal'
		end
	elseif infoType == 'dispel' then
		if info.isFailed then
			msgType = 'dispelFailed'
		else
			msgType = 'dispel'
		end
	elseif infoType == 'cast' then
		if info.isBegin then
			msgType = 'castBegin'
		elseif info.victim then
			msgType = 'castTargeted'
		else
			msgType = 'cast'
		end
	elseif infoType == 'honor' then
		if not info.amount then
			msgType = 'dishonor'
		elseif info.source then
			msgType = 'honorKill'
		else
			msgType = 'honor'
		end
	elseif info.type == 'reputation' then
		if info.rank then
			msgType = 'reputationRank'
		elseif info.isNegative then
			msgType = 'reputationMinus'
		else
			msgType = 'reputation'
		end
	elseif info.type == 'death' then
		if info.source then
			msgType = 'deathSource'
		elseif info.skill then
			msgType = 'deathSkill'
		else
			msgType = 'death'
		end
	elseif info.type == 'unknown' then
		ChatFrame1:AddMessage(string.format("%s: %s", event, info.message),1,0,0)
		return
	else
		msgType = infoType
	end

	-- Get necessary fields out of info table.
	local tokens = compost:Acquire()
	local fields = defaultFormats[msgType][2]
	for i, v in fields do
		tokens[v] = info[v]
	end
	
	-- Common conversion for all chat frames.
	if tokens.skill == ParserLib_MELEE then tokens.skill = loc["Melee"] end
	if tokens.skill == ParserLib_DAMAGESHIELD then tokens.skill = loc["Damage Shields"] end
	
	if infoType == 'hit' then
		if not info.element then
			tokens.element = 'physical'
		else
			tokens.element = self:GetElementType(info.element)
		end
	elseif infoType == 'heal' then
		tokens.element = 'heal'
	elseif infoType == 'buff' or infoType == 'debuff' then
		if info.amountRank then
			tokens.skill = string.format("%s(%d)", tokens.skill, info.amountRank)
		end
		tokens.element = infoType
	elseif infoType == 'durability' then
		if not tokens.item then tokens.item = loc["All Items"] end
	end

	for i, settings in self.settings do
		if settings.events and settings.events[event] and self:FilterCheck(i, info) then
			if infoType == 'hit' or infoType == 'environment' then
				tokens.trailers = self:GetTrailerString(i, info)
			end
			self:ParseInfo(i, msgType, tokens)
		end
	end
	
	compost:Reclaim(tokens)

end

function SimpleCombatLog:ParseInfo(id, msgType, tokens)
	
	local settings = self.settings[id]
	
	local myTokens = compost:Acquire()
	for i, v in tokens do
		myTokens[i] = v
	end
	
	if myTokens.source then 
		if myTokens.source == ParserLib_SELF then
			myTokens.source = self:Colorize(id, loc["You"], 'player' ) 
		else
			myTokens.source = self:Colorize(id, myTokens.source, self:GetUnitType(myTokens.source, true) ) 
		end
	end	
	if myTokens.victim then 
		if myTokens.victim == ParserLib_SELF then
			myTokens.victim = self:Colorize(id, loc["You"], 'player' ) 
		else
			myTokens.victim = self:Colorize(id, myTokens.victim, self:GetUnitType(myTokens.victim, true) ) 
		end
	end
	if myTokens.sourceGained then 
		if myTokens.sourceGained == ParserLib_SELF then
			myTokens.sourceGained = self:Colorize(id, loc["You"], 'player' ) 
		else
			myTokens.sourceGained = self:Colorize(id, myTokens.sourceGained, self:GetUnitType(myTokens.sourceGained, true) ) 
		end
	end	
	
	if myTokens.skill then
		if	settings.colorspell and myTokens.element then
			myTokens.skill = self:Colorize(id, myTokens.skill, myTokens.element)
		else
			myTokens.skill = self:Colorize(id, myTokens.skill, "skill")
		end
	end
	
	if myTokens.amount and myTokens.element then
		myTokens.amount = self:Colorize(id, myTokens.amount, myTokens.element)
	end
	
	local fields = defaultFormats[msgType][2]
	local fieldCount = table.getn(fields)
	
	local outputString
	if fieldCount == 0 then
		outputString = self:GetFormat(id, msgType)
	elseif fieldCount == 1 then
		outputString = string.format( self:GetFormat(id, msgType), myTokens[fields[1]] )
	elseif fieldCount == 2 then
		outputString = string.format( self:GetFormat(id, msgType), myTokens[fields[1]], myTokens[fields[2]] )
	elseif fieldCount == 3 then
		outputString = string.format( self:GetFormat(id, msgType), myTokens[fields[1]], myTokens[fields[2]], myTokens[fields[3]] )
	elseif fieldCount == 4 then
		outputString = string.format( self:GetFormat(id, msgType), myTokens[fields[1]], myTokens[fields[2]], myTokens[fields[3]], myTokens[fields[4]] )
	elseif fieldCount == 5 then
		outputString = string.format( self:GetFormat(id, msgType), myTokens[fields[1]], myTokens[fields[2]], myTokens[fields[3]], myTokens[fields[4]], myTokens[fields[5]] )
	elseif fieldCount == 6 then
		outputString = string.format( self:GetFormat(id, msgType), myTokens[fields[1]], myTokens[fields[2]], myTokens[fields[3]], myTokens[fields[4]], myTokens[fields[5]], myTokens[fields[6]] )
	elseif fieldCount == 7 then
		outputString = string.format( self:GetFormat(id, msgType), myTokens[fields[1]], myTokens[fields[2]], myTokens[fields[3]], myTokens[fields[4]], myTokens[fields[5]], myTokens[fields[6]], myTokens[fields[7]] )
	elseif fieldCount == 8 then
		outputString = string.format( self:GetFormat(id, msgType), myTokens[fields[1]], myTokens[fields[2]], myTokens[fields[3]], myTokens[fields[4]], myTokens[fields[5]], myTokens[fields[6]], myTokens[fields[7]], myTokens[fields[8]] )
	elseif fieldCount == 9 then
		outputString = string.format( self:GetFormat(id, msgType), myTokens[fields[1]], myTokens[fields[2]], myTokens[fields[3]], myTokens[fields[4]], myTokens[fields[5]], myTokens[fields[6]], myTokens[fields[7]], myTokens[fields[8]], myTokens[fields[9]] )
	end

--[[
	local color
	if self:GetValue(id, 'colorEvent') then	
		color = ChatTypeInfo[strsub(event,10)]
	end
	
	if color then
		self.chatFrames[id]:AddMessage(outputString, color.r, color.g, color.b)
	else
	end
]]	
	local chatframe = getglobal("ChatFrame" .. id)
	if chatframe then
		chatframe:AddMessage(outputString)
	end

	compost:Reclaim(myTokens)

	
end

function SimpleCombatLog:Colorize(id, value, valueType)
	
	if not valueType then return value end
	
	local r, g, b = self:GetColor(id, valueType)
	
	return string.format('|cff%02x%02x%02x%s|r', (r*255), (g*255), (b*255), value)

end

function SimpleCombatLog:GetValue(id, field)
	if not self.settings[id] then
		return false
	else
		return self.settings[id][field]
	end
end

function SimpleCombatLog:SetValue(id, field, value)

	if not self.settings[id] then
		self.settings[id] = {}
	else
		if self.settings[id].theme then
			self.settings[id] = CopyTable({}, self.settings[id])
			self.settings[id].theme = nil
			self.settingDB[id] = self.settings[id]
		end
	end
		
	self.settings[id][field] = value
	
end

function SimpleCombatLog:GetEvent(id, event)
	local db = self.settings[id]
	if not db or not db.events or not db.events[event] then
		return false
	else
		return db.events[event]
	end
end

function SimpleCombatLog:SetEvent(id, event, flag)
	if not self.settings[id] then
		self.settings[id] = {}
	else
		if self.settings[id].theme then
			self.settings[id] = CopyTable({}, self.settings[id])
			self.settings[id].theme = nil
			self.settingDB[id] = self.settings[id]
		end
	end

	
	if not self.settings[id].events then
		self.settings[id].events = {}
	end
	
	self.settings[id].events[event] = flag
	
end

SimpleCombatLog.filterFlags = {
	{
		player = 1,
		pet = 2,
		party = 3,
		raid = 4,
		target = 5,
		targettarget = 6,
		other = 7		
	},
	{
		player = 8,
		pet = 9,
		party = 10,
		raid = 11,
		target = 12,
		targettarget = 13,
		other = 14	
	}
}

function SimpleCombatLog:GetFilter(id, filterType, unitid, isSource)
	local db = self.settings[id]
	if not db or not db.filters or not db.filters[filterType] then
		return false
	else	
		local source
		if isSource then source = 1 else source = 2 end
		local flag = self.filterFlags[source][unitid]
		return db.filters[filterType][flag]
	end
end

function SimpleCombatLog:SetFilter(id, filterType, unitid, isSource, value)
	if not self.settings[id] then
		self.settings[id] = {}
	else
		if self.settings[id].theme then
			self.settings[id] = CopyTable({}, self.settings[id])
			self.settings[id].theme = nil
			self.settingDB[id] = self.settings[id]
		end
	end

	
	if not self.settings[id].filters then
		self.settings[id].filters = {}
	end
	
	if not self.settings[id].filters[filterType] then
		self.settings[id].filters[filterType] = {}
	end
	
	local source
	if isSource then source = 1 else source = 2 end
	local flag = self.filterFlags[source][unitid]
	
	self.settings[id].filters[filterType][flag] = value
end

function SimpleCombatLog:ToggleFilter(id, filterType, unitid, isSource)
	local value = self:GetFilter(id, filterType, unitid, isSource)
	if value then
		value = nil
	else
		value = true
	end
	self:SetFilter(id, filterType, unitid, isSource, value)

end



function SimpleCombatLog:SetAllFilters(id, unitid, isSource, value)
	for i, type in self.typeFilters do
		self:SetFilter(id, type, unitid, isSource, value)
	end
end

-- Return values:  r, g, b, isDefault
function SimpleCombatLog:GetColor(id, colorType)
	local color, isDefault
	if not self.settings[id] or not self.settings[id].colors or not self.settings[id].colors[colorType] then
		color = defaultColors[colorType]	
		isDefault = true
	else
		color = self.settings[id].colors[colorType]
		isDefault = false
	end
	return color.r, color.g, color.b, isDefault
end

function SimpleCombatLog:SetColor(id, colorType, r, g, b)
	if not self.settings[id] then
		self.settings[id] = {}
	else
		if self.settings[id].theme then
			self.settings[id] = CopyTable({}, self.settings[id])
			self.settings[id].theme = nil
			self.settingDB[id] = self.settings[id]
		end
	end

	
	if not self.settings[id].colors then
		self.settings[id].colors = {}
	end
	
	if not self.settings[id].colors[colorType] then
		self.settings[id].colors[colorType] = {}
	end
	
	self.settings[id].colors[colorType].r = r
	self.settings[id].colors[colorType].g = g
	self.settings[id].colors[colorType].b = b
end

function SimpleCombatLog:RestoreDefaultColors(id)
	if self.settings[id] and self.settings[id].colors then
		self.settings[id].colors = nil
	end
end

function SimpleCombatLog:GetFormat(id, msgType)
	local db = self.settings[id]
	if not db or not db.formats or not db.formats[msgType] then
		return defaultFormats[msgType][1], true
	else
		return db.formats[msgType], false
	end
end

function SimpleCombatLog:SetFormat(id, msgType, newFormat)
	if not self.settings[id] then
		self.settings[id] = {}
	else
		if self.settings[id].theme then
			self.settings[id] = CopyTable({}, self.settings[id])
			self.settings[id].theme = nil
			self.settingDB[id] = self.settings[id]
		end
	end

	
	if not self.settings[id].formats then
		self.settings[id].formats = {}
	end
	
	self.settings[id].formats[msgType] = newFormat	
end

function SimpleCombatLog:RestoreDefaultFormats(id)
	if self.settings[id] and self.settings[id].formats then
		self.settings[id].formats = nil
	end
end

function SimpleCombatLog:GetWatchList(id, watchType)
	local db = self.settings[id]
	if db and db.watchList and db.watchList[watchType] then
		return db.watchList[watchType]
	end
end

function SimpleCombatLog:AddWatch(id, watchType, keyword)
	if keyword == "%t" then keyword = UnitName('target') end

	if not keyword or keyword == "" then return end	
	if not self.settings[id] then
		self.settings[id] = {}
	else
		if self.settings[id].theme then
			self.settings[id] = CopyTable({}, self.settings[id])
			self.settings[id].theme = nil
			self.settingDB[id] = self.settings[id]
		end
	end

	
	if not self.settings[id].watchList then
		self.settings[id].watchList = {}
	end
	
	if not self.settings[id].watchList[watchType] then
		self.settings[id].watchList[watchType] = {}
	end
	
	self.settings[id].watchList[watchType][keyword] = true
end

function SimpleCombatLog:SetWatch(id, watchType, oldKeyword, newKeyword)
	if newKeyword == "%t" then newKeyword = UnitName('target') end
	if not newKeyword then return end
	if newKeyword ~= "" then
		self.settings[id].watchList[watchType][newKeyword] = true
	end
	self.settings[id].watchList[watchType][oldKeyword] = nil
end

function SimpleCombatLog:SaveTheme(id, themeName)
	if self.settings[id].theme then return end -- It is already a theme, noneed to save.
	
	local theme = self.settings[id]
	theme.theme = themeName
	
	self.themes[themeName] = theme
	self.themeDB[themeName] = theme
	
	self.settingDB[id] = themeName
	
	
end


function SimpleCombatLog:SetTheme(id, theme)
	local themeTable
	if type(theme) == 'string' then
		themeTable = self.themes[theme]
	else
		themeTable = theme
	end
	
	self.settingDB[id] = themeTable.theme
	self.settings[id] = themeTable
	self:RefreshSettings()
end

function SimpleCombatLog:RefreshSettings()

	local usedEvents = {}
	
	for event in self.events do
		for j in self.settings do			
			if self:GetEvent(j, event) then
				usedEvents[event] = true
				if not parser:IsEventRegistered("SimpleCombatLog", event) then
					parser:RegisterEvent("SimpleCombatLog", event, function(event, info) self:OnCombatEvent(event, info) end)
				end
			end
			
		end
	end

	
	-- Unregister non-used events from ParserLib.
	for event in self.events do
		if not usedEvents[event] and parser:IsEventRegistered("SimpleCombatLog", event) then
			parser:UnregisterEvent("SimpleCombatLog", event)
		end
	end
	

	for id, setting in self.settings do
		local chatframe = getglobal("ChatFrame" .. id)
		if chatframe then
			if setting.resize then				
				chatframe:SetMaxResize(1000, 700)
				chatframe:SetMinResize(40, 40)	
			else
				chatframe:SetMaxResize(608, 400)
				chatframe:SetMinResize(296, 75)		
			end
		end
	end
		
	

end



-- Get constant element string from localized element string.
function SimpleCombatLog:GetElementType(element)
	if element == SPELL_SCHOOL0_CAP then return 'physical'
	elseif element == SPELL_SCHOOL1_CAP then return 'holy'
	elseif element == SPELL_SCHOOL2_CAP then return 'fire'
	elseif element == SPELL_SCHOOL3_CAP then return 'nature'
	elseif element == SPELL_SCHOOL4_CAP then return 'frost'
	elseif element == SPELL_SCHOOL5_CAP then return 'shadow'
	elseif element == SPELL_SCHOOL6_CAP then return 'arcane'
	else return element end
end

-- Get trailer string by parsing info.
function SimpleCombatLog:GetTrailerString(id, info)
	local trailer = ''
	if info.isCrushing then trailer = trailer .. self:GetFormat(id, 'crushing')end
	if info.isGlancing then trailer = trailer .. self:GetFormat(id, 'glancing')end
	if info.amountAbsorb then trailer = trailer .. format(self:GetFormat(id, 'absorb'), info.amountAbsorb) end
	if info.amountResist then trailer = trailer .. format(self:GetFormat(id, 'resist'), info.amountResist) end
	if info.amountBlock then trailer = trailer .. format(self:GetFormat(id, 'block'), info.amountBlock) end
	if info.amountVulnerable then trailer = trailer .. format(self:GetFormat(id, 'vulnerable'), info.amountVulnerable) end
	return trailer
end

-- Check for the filter settings, return true if this message can be shown, false if the message is filtered.
function SimpleCombatLog:FilterCheck(id, info)
	
	-- These message ignore name filters.
	if info.type == 'experience'
	or info.type == 'honor'
	or info.type == 'reputation' 
	or info.type == 'fail' then
		return true
	end	
	
	if not self.settings[id] then
		return true
	end
	
	local watch = self:GetWatchList(id, 'skill')
	
	-- Custom skill watch?
	if watch and info.skill and watch[info.skill] then
		return true
	end
	
	
	-- Checking names
	if not info.source and not info.victim then return true end
	
	if info.source then
		local name = info.source
		
		-- Custom name watch.
		watch = self:GetWatchList(id, 'source')
		if watch and watch[name]  then return true end
		
		if self:GetFilter(id, info.type, self:GetUnitType(name), true) then
			return true
		end
		if self:IsTarget(name) and self:GetFilter(id, info.type, 'target', true) then
			return true
		end
		if self:IsTargetTarget(name) and self:GetFilter(id, info.type, 'targettarget', true) then
			return true
		end
	end
	
	if info.victim then
		local name = info.victim
		
		-- Custom name watch.
		watch = self:GetWatchList(id, 'victim')
		if watch and watch[name]  then return true end
		
		if self:GetFilter(id, info.type, self:GetUnitType(name)) then
			return true
		end
		if self:IsTarget(name) and self:GetFilter(id, info.type, 'target') then
			return true
		end
		if self:IsTargetTarget(name) and self:GetFilter(id, info.type, 'targettarget') then
			return true
		end
	end
	
	-- The message does not match any filter setting, block.
	return false
	
end

function SimpleCombatLog:IsTarget(name)
	return name == UnitName("target")
end

function SimpleCombatLog:IsTargetTarget(name)
	return name == UnitName("targettarget")
end


-- Get what is this name : player, pet, party, raid, or others.
function SimpleCombatLog:GetUnitType(name, checkTarget)
	if not name then return end
	
	if checkTarget then
		if name == UnitName('target') then return 'target' end
		if name == UnitName('targettarget') then return 'targettarget' end
	end
	
	if name == ParserLib_SELF then return 'player' end
		
	-- This is NOT player, but someone which has an identical name to player ( most likely a pet name).
	if name == UnitName('player') then return 'other' end	

	local unit = roster:GetUnitIDFromName(name)
	
	if not unit then return 'other' end
	
	if unit == 'pet' then return 'pet' end
	
	if string.find(unit, 'party', 1, true) then return 'party' end
	
	return 'raid'
	
end


function SimpleCombatLog:LoadInitialSettings()
	SimpleCombatLogDB = nil
	SimpleCombatLogDBPerChar = nil
	self:InitGlobalDB()
	self:InitPerCharDB()
	self.settings = {}
	self:SetTheme(2, 'default')
	self:Print("Loaded default settings (custom themes will exist until next login).")
end

function SimpleCombatLog:LoadSettingMenu(id)
--[[
	if not IsAddOnLoaded(optionTitle) then
		local loaded, reason = LoadAddOn(optionTitle)
		if not loaded then
			self:Print("Failed to load option UI, reason: ", reason)
			return
		end
	end
]]
	self:ShowSettingMenu(id)
end



-- arg1: the format string
-- arg2, arg3, ...  : the tokens to replace with.
function SimpleCombatLog:ConvertFormat(...)
	local msg = string.gsub(table.remove(arg, 1), "|", "||") -- "add slash" so that users can see color codes.	
	for i in arg do
		arg[i] = string.format("#%s#", arg[i])
	end
	return string.format(msg, unpack(arg) )
end

-- arg1: the format string
-- arg2, arg3, ...  : the tokens to replace with.
function SimpleCombatLog:UnconvertFormat(...)
	local msg = string.gsub(table.remove(arg, 1), "||", "|") -- "strip slash" to convert back to actual color codes.
	local n = table.getn(arg)
	for i in ipairs(arg) do
		arg[i] = string.format("#%s#", arg[i])
		msg = string.gsub(msg, arg[i], string.format("%%%%%d$s", i) )
	end
	return msg
end

function SimpleCombatLog:LoadMenuChatTypeGroups(id, title, chatTypeGroups)
	dewdrop:AddLine(
		'text', title,
		'isTitle', true		
	)
	
	for i, v in chatTypeGroups do
		local eventName = ChatTypeGroup[v][1]
		local eventDesc = getglobal(eventName)
		dewdrop:AddLine(
			'text', eventDesc,
			'func', function(id, event) 
				self:SetEvent(id, event, not self:GetEvent(id, event) )
				self:RefreshSettings()
			end,
			'arg1', id,
			'arg2', eventName,
			'checked', self:GetEvent(id, eventName)
		)
	end		
end


function SimpleCombatLog:ClearSettings(id, noRefresh)
	if not self.settings[id] then return end
	
	local colors = self.settings[id].colors
	local formats = self.settings[id].formats
	
	self.settings[id] = nil
	
	if colors or formats then
		self.settings[id] = {
			colors = colors,
			formats = formats,
		}
	end

	self.settingDB[id] = self.settings[id]
	
	if not noRefresh then
		self:RefreshSettings()
	end
	
end


function SimpleCombatLog:ShowSettingMenu(id)

	local tabFrame = getglobal("ChatFrame" .. id .. "Tab")	
	if not tabFrame then return end

	if not self.initGUI then
		self.typeList = {
			"hit", 	"hitCrit", 	"hitDOT", 	"heal", 	"healCrit", 	"healDOT", 	"miss", 	"gain", 	"drain", 	"leech", "environment", "",
			"buff", 	"debuff", 	"fade", 	"dispel", 	"dispelFailed", 	"extraattack", 	"cast", 	"castBegin", 	"castTargeted", "interrupt", "",
			"create", 	"death", 	"deathSource", "deathSkill", "honor", 	"honorKill", "dishonor", "experience", 	"reputation",  "reputationRank", "reputationMinus", "enchant", 	"feedpet", 	"fail",
		}
	
		self.colorList = {
		"player", 	"pet", 	"raid", 	"party", 	"target", 	"targettarget", 	"other", "",
		"physical", 	"holy", 	"fire", 	"nature", 	"frost", 	"shadow", 	"arcane", "",
		"heal", 	"miss", 	"buff", 	"debuff", 	"skill", 		
		}
		
		if not dewdrop then
			dewdrop = AceLibrary("Dewdrop-2.0")
		end

		self.initGUI = true
		
	end

	if not dewdrop:IsRegistered(tabFrame) then
		dewdrop:Register(tabFrame, 
			'children', function(level, value) SimpleCombatLog:MenuSettings(id, level, value) end, 
			'dontHook', true
		)
	end
		
	dewdrop:Open(tabFrame)
end


function SimpleCombatLog:MenuSettings(id, level, value)
	
--	if 1 then dewdrop:FeedAceOptionsTable( self:GetOptionsTable(id) )  		return  end
	
	if level == 1 then

		dewdrop:AddLine(
			'text', string.format( loc["SCL: ChatFrame%d Settings"], id),
			'isTitle', true,
			'notCheckable', true
		)

		dewdrop:AddLine(
			'text', loc["Filters"],
			'hasArrow', true,
			'value', 'filter',
			'notCheckable', true
		)
		
		dewdrop:AddLine(
			'text', loc["Events"],
			'hasArrow', true,
			'value', 'event',
			'notCheckable', true
		)		

		dewdrop:AddLine(
			'text', loc["Colors"],
			'notCheckable', true,
			'hasArrow', true,
			'value', 'color'
		)		
		
		dewdrop:AddLine(
			'text', loc["Formats"],
			'notCheckable', true,
			'hasArrow', true,
			'value', 'format'
		)		
		
		dewdrop:AddLine(
			'text', loc["Custom watch"],
			'notCheckable', true,
			'hasArrow', true,
			'value', 'watch'
		)
			
		 -- dewdrop:AddLine()

		dewdrop:AddLine(
			'text', loc["Suppress combat log"],
			'func', function() 
				self:SetValue(id, 'suppress', not self:GetValue(id, 'suppress') ) 
				self:RefreshSettings()
			end,
			'checked', self:GetValue(id, 'suppress')
		)

		dewdrop:AddLine(
			'text', loc["Color skills by element"],
			'func', function() 
				self:SetValue(id, 'colorspell', not self:GetValue(id, 'colorspell') ) 
				self:RefreshSettings()
			end,
			'checked', self:GetValue(id, 'colorspell')
		)

		dewdrop:AddLine(
			'text', loc["Greater resize"],
			'func', function() 
				self:SetValue(id, 'resize', not self:GetValue(id, 'resize') ) 
				self:RefreshSettings()
			end,
			'checked', self:GetValue(id, 'resize')
		)
		
		
		dewdrop:AddLine()
		
		dewdrop:AddLine(
			'text', loc["Clear all settings"],
			'notCheckable', true,
			'func', function() self:ClearSettings(id) end
		)
		
		dewdrop:AddLine(
			'text', loc["Load Theme"],
			'hasArrow', true,
			'notCheckable', true,
			'value', 'LoadTheme'
		)

		
		if not self:GetValue(id, 'theme') then
			dewdrop:AddLine(
				'text', loc["Save Theme"],
				'hasArrow', true,
				'notCheckable', true,
				'value', 'SaveTheme'
			)
		end
		
		dewdrop:AddLine(
			'text', loc["Delete Theme"],
			'hasArrow', true,
			'notCheckable', true,
			'value', 'DeleteTheme'
		)


	elseif level == 2 then
	
		if value == 'color' then
			local c, d, title
			for i, v in self.colorList do
				if v ~= "" then
					local r, g, b, isDefault = self:GetColor(id, v)
					if not isDefault then title = "|cffeda55f".. loc[v] .."|r" else title = loc[v] end
					dewdrop:AddLine(
						'text', title,
						'hasColorSwatch', true,
						'r', r,
						'g', g,
						'b', b,
						'notCheckable', true,
						'colorFunc', function(id, colorType, r, g, b) self:SetColor(id, colorType, r, g, b) end,
						'colorArg1', id,
						'colorArg2', v
					)
				else
					dewdrop:AddLine()
				end
			end
			
			dewdrop:AddLine()
			dewdrop:AddLine(
				'text', loc["Restore default colors"],
				'notCheckable', true,
				'func', function() self:RestoreDefaultColors(id) end
			)
			
		elseif value == 'event' then
			
			dewdrop:AddLine(
				'text', loc["Events"],
				'isTitle', true,
				'notCheckable', true
			)				
			-- I decided not to bother messing with the event groups, just use Blizzard's groupings.
			dewdrop:AddLine(
				'text', COMBAT_MESSAGES,
				'notCheckable', true,
				'hasArrow', true,
				'value', COMBAT_MESSAGES
			)		
			dewdrop:AddLine(
				'text', SPELL_MESSAGES,
				'notCheckable', true,
				'hasArrow', true,
				'value', SPELL_MESSAGES
			)		
			dewdrop:AddLine(
				'text', SPELL_OTHER_MESSAGES,
				'notCheckable', true,
				'hasArrow', true,
				'value', SPELL_OTHER_MESSAGES
			)		
			dewdrop:AddLine(
				'text', PERIODIC_MESSAGES,
				'notCheckable', true,
				'hasArrow', true,
				'value', PERIODIC_MESSAGES
			)		

		elseif value == 'format' then
		
			local msgType, formats, isDefault

			for i, v in self.typeList do
				if v ~= "" then
					msgType = v
					formats, isDefault = self:GetFormat(id, v)
					if not isDefault then msgType = "|cffeda55f"..msgType.."|r" end
					dewdrop:AddLine(
						'text', msgType,
						'notCheckable', true,
						'hasArrow', true,
						'hasEditBox', true,
						'editBoxText', self:ConvertFormat(formats, unpack(defaultFormats[v][2])),
						'editBoxFunc', function(id, msgType, msgFormat) self:SetFormat(id, msgType, self:UnconvertFormat(msgFormat, unpack(defaultFormats[msgType][2]))) end,
						'editBoxArg1', id,
						'editBoxArg2', v
					)
				else
					dewdrop:AddLine()
				end
			end


			dewdrop:AddLine()
			dewdrop:AddLine(
				'text', loc["Restore default formats"],
				'notCheckable', true,
				'func', function() self:RestoreDefaultFormats(id) end
			)
				
		elseif value == 'filter' then
		
			dewdrop:AddLine(
				'text', loc["Type Menu Title"],
				'isTitle', true,
				'notCheckable', true
			)
			
			dewdrop:AddLine(
				'text', loc["AllFilter"],
				'hasArrow', true,
				'value', 'AllFilter',
				'notCheckable', true
			)
				
			
			for i, v in self.typeFilters do
					dewdrop:AddLine(
						'text', loc[v],
						'hasArrow', true,
						'tooltipTitle', loc[v],
						'tooltipText', loc.typesTooltip[v],
						'value', v,
						'notCheckable', true
					)
			end
		
		elseif value == 'watch' then
			local list
			
			dewdrop:AddLine(
				'text', loc["Source Watch"],
				'isTitle', true,
				'notCheckable', true
			)
			list = self:GetWatchList(id, 'source')
			if list then
				for keyword, watchType in list do
					local keyword = keyword
					dewdrop:AddLine(
						'text', keyword,
						'hasArrow', true,
						'notCheckable', true,
						'hasEditBox', true,
						'editBoxText', keyword,
						'editBoxFunc', function(text) self:SetWatch(id, 'source', keyword, text) end
					)
				end
			end
			dewdrop:AddLine(
				'text', loc["Add source watch"],
				'hasArrow', true,
				'notCheckable', true,
				'hasEditBox', true,
				'editBoxFunc', function(text) self:AddWatch(id, 'source', text) end
			)
			
			dewdrop:AddLine()
			
			dewdrop:AddLine(
				'text', loc["Victim Watch"],
				'isTitle', true,
				'notCheckable', true
			)
			list = self:GetWatchList(id, 'victim')
			if list then
				for keyword, watchType in list do
					local keyword = keyword
					dewdrop:AddLine(
						'text', keyword,
						'hasArrow', true,
						'notCheckable', true,
						'hasEditBox', true,
						'editBoxText', keyword,
						'editBoxFunc', function(text) self:SetWatch(id, 'victim', keyword, text) end
					)
				end
			end
			dewdrop:AddLine(
				'text', loc["Add victim watch"],
				'hasArrow', true,
				'notCheckable', true,
				'hasEditBox', true,
				'editBoxFunc', function(text) self:AddWatch(id, 'victim', text) end
			)

			dewdrop:AddLine()

			
			dewdrop:AddLine(
				'text', loc["Skill Watch"],
				'isTitle', true,
				'notCheckable', true
			)
			list = self:GetWatchList(id, 'skill')
			if list then
				for keyword, watchType in list do
					local keyword = keyword
					dewdrop:AddLine(
						'text', keyword,
						'hasArrow', true,
						'notCheckable', true,
						'hasEditBox', true,
						'editBoxText', keyword,
						'editBoxFunc', function(text) self:SetWatch(id, 'skill', keyword, text) end
					)
				end
			end
			dewdrop:AddLine(
				'text', loc["Add skill watch"],
				'hasArrow', true,
				'notCheckable', true,
				'hasEditBox', true,
				'editBoxFunc', function(text) self:AddWatch(id, 'skill', text) end
			)
			
		elseif value == 'LoadTheme' then
			for themeName, themeTable in self.themes do
				local themeName = themeName
				local themeTable = themeTable
				local title
				if self.themeDB[themeName] then
					title = string.format("|cffeda55f%s|r", themeName)
				else
					title = themeName
				end
					
				dewdrop:AddLine(
					'text', title,
					'func', function() self:SetTheme(id, themeTable) end,
					'checked', (self:GetValue(id, 'theme') == themeName )
				)
			end			
		
		elseif value == 'SaveTheme' then
			for themeName, themeTable in self.themes do
				local themeName = themeName
				local themeTable = themeTable
				dewdrop:AddLine(
					'text', themeName,
					'func', function() self:SaveTheme(id, themeName) end
				)
			end	
			dewdrop:AddLine(
				'text', loc["Add new Theme"],
				'hasArrow', true,
				'hasEditBox', true,
				'editBoxFunc', function(text) self:SaveTheme(id, text) end
			)
		
		elseif value == 'DeleteTheme' then
			dewdrop:AddLine(
				'text', "Not yet implemented"
			)
		end
		
	elseif level == 3 then
	  		
		if value == COMBAT_MESSAGES then
			self:LoadMenuChatTypeGroups(id, COMBAT_MESSAGES, CombatLogMenuChatTypeGroups)
		elseif value == SPELL_MESSAGES then
			self:LoadMenuChatTypeGroups(id, SPELL_MESSAGES, SpellLogMenuChatTypeGroups)
		elseif value == SPELL_OTHER_MESSAGES then
			self:LoadMenuChatTypeGroups(id, SPELL_OTHER_MESSAGES, SpellLogOtherMenuChatTypeGroups)
		elseif value == PERIODIC_MESSAGES then
			self:LoadMenuChatTypeGroups(id, PERIODIC_MESSAGES, PeriodicLogMenuChatTypeGroups)
		elseif value == 'AllFilter' then
		
			dewdrop:AddLine(
				'text', string.format(loc["Name Menu Title"], loc["AllFilter"]),
				'isTitle', true,
				'notCheckable', true
			)
			

			dewdrop:AddLine(
				'text', loc["Source"],
				'isTitle', true
			)
						
			for i, name in self.nameFilters do
				local name = name
				local state	= nil -- 0: none, 1: partial, 2: all
				for j, type in self.typeFilters do
					local checked = self:GetFilter(id, type, name, true)
					if not state then 
						if checked then 
							state = 2
						else
							state = 0
						end
					elseif state == 0 and checked then
						state = 1
					elseif state == 2 and not checked then
						state = 1
					end
				end
				local checked, checkIcon, func
				if state == 0 then
					checked = false
					checkIcon = nil					
					func = function() self:SetAllFilters(id, name, true, true) end
				elseif state == 1 then
					checked = true
					checkIcon = "Interface\\Buttons\\UI-CheckBox-Check-Disabled"
					func = function() self:SetAllFilters(id, name, true, true) end
				else
					checked = true
					checkIcon = nil
					func = function() self:SetAllFilters(id, name, true, nil) end
				end
				dewdrop:AddLine(
					'text', loc[name],
					'checked', checked,
					'checkIcon', checkIcon,
					'func', func
				)
			end


			dewdrop:AddLine(
				'text', loc["Victim"],
				'isTitle', true
			)			
			for i, name in self.nameFilters do
				local name = name
				local state	= nil -- 0: none, 1: partial, 2: all
				for j, type in self.typeFilters do
					local checked = self:GetFilter(id, type, name, false)
					if not state then 
						if checked then 
							state = 2
						else
							state = 0
						end
					elseif state == 0 and checked then
						state = 1
					elseif state == 2 and not checked then
						state = 1
					end
				end
				local checked, checkIcon, func
				if state == 0 then
					checked = false
					checkIcon = nil
					func = function() self:SetAllFilters(id, name, false, true) end
				elseif state == 1 then
					checked = true
					checkIcon = "Interface\\Buttons\\UI-CheckBox-Check-Disabled"
					func = function() self:SetAllFilters(id, name, false, true) end
				else
					checked = true
					checkIcon = nil
					func = function() self:SetAllFilters(id, name, false, nil) end
				end
				dewdrop:AddLine(
					'text', loc[name],
					'checked', checked,
					'checkIcon', checkIcon,
					'func', func
				)
			end
			
			
			
		else
	
			dewdrop:AddLine(
				'text', string.format(loc["Name Menu Title"], loc[value]),
				'isTitle', true,
				'notCheckable', true
			)
			
			
			dewdrop:AddLine(
				'text', loc["Source"],
				'isTitle', true
			)
				
			for i, v in self.nameFilters do
				dewdrop:AddLine(
					'text', loc[v],
					'func', function(filterType, unitid) self:ToggleFilter(id, value, unitid, true) end,
					'arg1', value,
					'arg2', v,
					'checked', self:GetFilter(id, value, v, true)
				)				
			end
			

			dewdrop:AddLine(
				'text', loc["Victim"],
				'isTitle', true
			)
			
			for i, v in self.nameFilters do
				dewdrop:AddLine(
					'text', loc[v],
					'func', function(filterType, unitid) self:ToggleFilter(id, value, unitid) end,
					'arg1', value,
					'arg2', v,
					'checked', self:GetFilter(id, value, v)
				)				
			end	

		end
	end
		


end


